"""The display commands module.

These commands are used in the following models:
AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DISPlay:WINDow1:STATe <display_state>
    - DISPlay:WINDow1:STATe?
    - DISPlay:WINDow2:STATe <display_state>
    - DISPlay:WINDow2:STATe?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class DisplayWindow2State(SCPICmdWrite, SCPICmdRead):
    """The ``DISPlay:WINDow2:STATe`` command.

    Description:
        - This command minimizes or restores the sequence or waveform window of the arbitrary
          waveform generator. This command only minimizes or restores the display area; it does not
          close the window. There is no maximizing. WINDow1 - Sequence window WINDow2 - Waveform
          window

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay:WINDow2:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow2:STATe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DISPlay:WINDow2:STATe value`` command.

    SCPI Syntax:
        ```
        - DISPlay:WINDow2:STATe <display_state>
        - DISPlay:WINDow2:STATe?
        ```

    Info:
        - ``<display_state>`` ::=<Boolean>.
        - ``0`` indicates False, minimizes the window display.
        - ``1`` indicates True, restores the window display.
    """


class DisplayWindow2(SCPICmdRead):
    """The ``DISPlay:WINDow2`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay:WINDow2?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow2?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``DISPlay:WINDow2:STATe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = DisplayWindow2State(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> DisplayWindow2State:
        """Return the ``DISPlay:WINDow2:STATe`` command.

        Description:
            - This command minimizes or restores the sequence or waveform window of the arbitrary
              waveform generator. This command only minimizes or restores the display area; it does
              not close the window. There is no maximizing. WINDow1 - Sequence window WINDow2 -
              Waveform window

        Usage:
            - Using the ``.query()`` method will send the ``DISPlay:WINDow2:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow2:STATe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DISPlay:WINDow2:STATe value``
              command.

        SCPI Syntax:
            ```
            - DISPlay:WINDow2:STATe <display_state>
            - DISPlay:WINDow2:STATe?
            ```

        Info:
            - ``<display_state>`` ::=<Boolean>.
            - ``0`` indicates False, minimizes the window display.
            - ``1`` indicates True, restores the window display.
        """
        return self._state


class DisplayWindow1State(SCPICmdWrite, SCPICmdRead):
    """The ``DISPlay:WINDow1:STATe`` command.

    Description:
        - This command minimizes or restores the sequence or waveform window of the arbitrary
          waveform generator. This command only minimizes or restores the display area; it does not
          close the window. There is no maximizing. WINDow1 - Sequence window WINDow2 - Waveform
          window

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay:WINDow1:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow1:STATe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DISPlay:WINDow1:STATe value`` command.

    SCPI Syntax:
        ```
        - DISPlay:WINDow1:STATe <display_state>
        - DISPlay:WINDow1:STATe?
        ```

    Info:
        - ``<display_state>`` ::=<Boolean>.
        - ``0`` indicates False, minimizes the window display.
        - ``1`` indicates True, restores the window display.
    """


class DisplayWindow1(SCPICmdRead):
    """The ``DISPlay:WINDow1`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay:WINDow1?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow1?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``DISPlay:WINDow1:STATe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = DisplayWindow1State(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> DisplayWindow1State:
        """Return the ``DISPlay:WINDow1:STATe`` command.

        Description:
            - This command minimizes or restores the sequence or waveform window of the arbitrary
              waveform generator. This command only minimizes or restores the display area; it does
              not close the window. There is no maximizing. WINDow1 - Sequence window WINDow2 -
              Waveform window

        Usage:
            - Using the ``.query()`` method will send the ``DISPlay:WINDow1:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow1:STATe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DISPlay:WINDow1:STATe value``
              command.

        SCPI Syntax:
            ```
            - DISPlay:WINDow1:STATe <display_state>
            - DISPlay:WINDow1:STATe?
            ```

        Info:
            - ``<display_state>`` ::=<Boolean>.
            - ``0`` indicates False, minimizes the window display.
            - ``1`` indicates True, restores the window display.
        """
        return self._state


class Display(SCPICmdRead):
    """The ``DISPlay`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.window1``: The ``DISPlay:WINDow1`` command tree.
        - ``.window2``: The ``DISPlay:WINDow2`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DISPlay") -> None:
        super().__init__(device, cmd_syntax)
        self._window1 = DisplayWindow1(device, f"{self._cmd_syntax}:WINDow1")
        self._window2 = DisplayWindow2(device, f"{self._cmd_syntax}:WINDow2")

    @property
    def window1(self) -> DisplayWindow1:
        """Return the ``DISPlay:WINDow1`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DISPlay:WINDow1?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow1?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``DISPlay:WINDow1:STATe`` command.
        """
        return self._window1

    @property
    def window2(self) -> DisplayWindow2:
        """Return the ``DISPlay:WINDow2`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DISPlay:WINDow2?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay:WINDow2?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``DISPlay:WINDow2:STATe`` command.
        """
        return self._window2



"""The save_and_recall commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC,
DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *RCL <NR1>
    - *SAV <NR1>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Sav(SCPICmdWrite):
    """The ``*SAV`` command.

    Description:
        - Stores the state of the oscilloscope to a specified memory location. You can use the
          ``*RCL`` command to restore the oscilloscope to this saved state at a later time.

    Usage:
        - Using the ``.write(value)`` method will send the ``*SAV value`` command.

    SCPI Syntax:
        ```
        - *SAV <NR1>
        ```

    Info:
        - ``<NR1>`` specifies a location in which to save the state of the oscilloscope. Location
          values range from 1 through 10. Using an out-of-range location value causes an execution
          error. Any settings that have been stored previously at this location will be overwritten.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*SAV") -> None:
        super().__init__(device, cmd_syntax)


class Rcl(SCPICmdWrite):
    """The ``*RCL`` command.

    Description:
        - This command restores the state of the oscilloscope from a copy of the settings stored in
          memory (The settings are stored using the ``*SAV`` command).

    Usage:
        - Using the ``.write(value)`` method will send the ``*RCL value`` command.

    SCPI Syntax:
        ```
        - *RCL <NR1>
        ```

    Info:
        - ``<NR1>`` is a value in the range from 1 to 10, which specifies a saved setup storage
          location.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*RCL") -> None:
        super().__init__(device, cmd_syntax)



"""SMU Model 26xx device driver module."""

import string

from abc import ABC
from typing import Tuple, Union

from tm_devices.commands import (
    SMU2601BCommands,
    SMU2601BPulseCommands,
    SMU2602BCommands,
    SMU2604BCommands,
    SMU2606BCommands,
    SMU2611BCommands,
    SMU2612BCommands,
    SMU2614BCommands,
    SMU2634BCommands,
    SMU2635BCommands,
    SMU2636BCommands,
    SMU2651ACommands,
    SMU2657ACommands,
)
from tm_devices.driver_mixins.device_control import TSPControl
from tm_devices.driver_mixins.shared_implementations.common_tsp_error_check_mixin import (
    CommonTSPErrorCheckMixin,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.source_measure_units.source_measure_unit import SourceMeasureUnit
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813


@family_base_class
class SMU26xx(CommonTSPErrorCheckMixin, TSPControl, SourceMeasureUnit, ABC):
    """Base SMU26xx device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(string.ascii_lowercase[: self.total_channels])

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        # Grab the total channel count based on whether the last digit in the model is even/odd
        return 2 if (not int([x for x in self.model if x.isdigit()][-1]) % 2) else 1

    @cached_property
    def commands(
        self,
    ) -> Union[
        SMU2601BCommands,
        SMU2602BCommands,
        SMU2604BCommands,
        SMU2606BCommands,
        SMU2611BCommands,
        SMU2612BCommands,
        SMU2614BCommands,
        SMU2634BCommands,
        SMU2635BCommands,
        SMU2636BCommands,
        SMU2601BPulseCommands,
        SMU2651ACommands,
        SMU2657ACommands,
    ]:
        """Return the device commands."""
        return super().commands

    ################################################################################################
    # Public Methods
    ################################################################################################

    ################################################################################################
    # Private Methods
    ################################################################################################



"""The tstamptable commands module.

These commands are used in the following models:
LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - TSTamptable:ADDNew <QString>
    - TSTamptable:DELETE <QString>
    - TSTamptable:LIST?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class TstamptableList(SCPICmdRead):
    """The ``TSTamptable:LIST`` command.

    Description:
        - This command queries the list of History time stamp results tables has been added.

    Usage:
        - Using the ``.query()`` method will send the ``TSTamptable:LIST?`` query.
        - Using the ``.verify(value)`` method will send the ``TSTamptable:LIST?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - TSTamptable:LIST?
        ```
    """


class TstamptableDelete(SCPICmdWrite):
    """The ``TSTamptable:DELETE`` command.

    Description:
        - This command deletes the requested History time stamp results table.

    Usage:
        - Using the ``.write(value)`` method will send the ``TSTamptable:DELETE value`` command.

    SCPI Syntax:
        ```
        - TSTamptable:DELETE <QString>
        ```

    Info:
        - ``<QString>`` specifies the History time stamp results table name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class TstamptableAddnew(SCPICmdWrite):
    """The ``TSTamptable:ADDNew`` command.

    Description:
        - This command adds a new History time stamp results table to the scope application.

    Usage:
        - Using the ``.write(value)`` method will send the ``TSTamptable:ADDNew value`` command.

    SCPI Syntax:
        ```
        - TSTamptable:ADDNew <QString>
        ```

    Info:
        - ``<QString>`` specifies the History time stamp results table name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Tstamptable(SCPICmdRead):
    """The ``TSTamptable`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``TSTamptable?`` query.
        - Using the ``.verify(value)`` method will send the ``TSTamptable?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.addnew``: The ``TSTamptable:ADDNew`` command.
        - ``.delete``: The ``TSTamptable:DELETE`` command.
        - ``.list``: The ``TSTamptable:LIST`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "TSTamptable"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._addnew = TstamptableAddnew(device, f"{self._cmd_syntax}:ADDNew")
        self._delete = TstamptableDelete(device, f"{self._cmd_syntax}:DELETE")
        self._list = TstamptableList(device, f"{self._cmd_syntax}:LIST")

    @property
    def addnew(self) -> TstamptableAddnew:
        """Return the ``TSTamptable:ADDNew`` command.

        Description:
            - This command adds a new History time stamp results table to the scope application.

        Usage:
            - Using the ``.write(value)`` method will send the ``TSTamptable:ADDNew value`` command.

        SCPI Syntax:
            ```
            - TSTamptable:ADDNew <QString>
            ```

        Info:
            - ``<QString>`` specifies the History time stamp results table name.
        """
        return self._addnew

    @property
    def delete(self) -> TstamptableDelete:
        """Return the ``TSTamptable:DELETE`` command.

        Description:
            - This command deletes the requested History time stamp results table.

        Usage:
            - Using the ``.write(value)`` method will send the ``TSTamptable:DELETE value`` command.

        SCPI Syntax:
            ```
            - TSTamptable:DELETE <QString>
            ```

        Info:
            - ``<QString>`` specifies the History time stamp results table name.
        """
        return self._delete

    @property
    def list(self) -> TstamptableList:
        """Return the ``TSTamptable:LIST`` command.

        Description:
            - This command queries the list of History time stamp results tables has been added.

        Usage:
            - Using the ``.query()`` method will send the ``TSTamptable:LIST?`` query.
            - Using the ``.verify(value)`` method will send the ``TSTamptable:LIST?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - TSTamptable:LIST?
            ```
        """
        return self._list
